/****************************************************************************
 ****************************************************************************/

#include "jwm.h"

int pagerHeight;
int pagerWidth;

static int pagerDeskWidth;

static double scalex, scaley;
static Pixmap pager;
static GC pagerGC;

static void DrawPager();
static void DrawPagerClient(ClientNode *np);

/****************************************************************************
 ****************************************************************************/
void InitializePager() {

	pagerHeight = TRAY_HEIGHT - 2;
	pagerDeskWidth = (pagerHeight * rootWidth) / rootHeight;

	pagerWidth = (pagerDeskWidth + 1) * desktopCount;

	scalex = (double)pagerDeskWidth / (double)rootWidth;
	scaley = (double)pagerHeight / (double)rootHeight;

	pager = XCreatePixmap(display, rootWindow,
		(pagerDeskWidth + 1) * desktopCount,
		pagerHeight, DefaultDepth(display, 0));
	pagerGC = XCreateGC(display, pager, 0, NULL);

}

/****************************************************************************
 ****************************************************************************/
void DestroyPager() {
	XFreeGC(display, pagerGC);
	XFreePixmap(display, pager);
}

/****************************************************************************
 ****************************************************************************/
Pixmap GetPager() {
	DrawPager();
	return pager;
}

/****************************************************************************
 ****************************************************************************/
void DrawPager() {
	ClientNode *np;
	int x;

	XSetForeground(display, pagerGC, colors[COLOR_BORDER]);
	XFillRectangle(display, pager, pagerGC, 0, 0,
		(pagerDeskWidth + 1) * desktopCount, pagerHeight);

	XSetForeground(display, pagerGC, colors[COLOR_ACTIVE_BORDER]);
	XFillRectangle(display, pager, pagerGC,
		currentDesktop * (pagerDeskWidth + 1), 0,
		pagerDeskWidth, pagerHeight);

	/* Draw the window outlines */
	for(x = LAYER_BOTTOM; x <= LAYER_TOP; x++) {
		for(np = nodeTail; np; np = np->prev) {
			if(np->layer == x) {
				DrawPagerClient(np);
			}
		}
	}

	/* Draw the pager outline. */
	XSetForeground(display, pagerGC, colors[COLOR_BORDER_DOWN]);

	for(x = 1; x < desktopCount; x++) {
		XDrawLine(display, pager, pagerGC, (pagerDeskWidth + 1) * x - 1, 0,
			(pagerDeskWidth + 1) * x - 1, pagerHeight - 1);
	}

	XDrawRectangle(display, pager, pagerGC, 0, 0,
		(pagerDeskWidth + 1) * desktopCount - 1, pagerHeight - 1);

}

/****************************************************************************
 ****************************************************************************/
void DrawPagerClient(ClientNode *np) {
	int x, y, width, height;
	int deskOffset;

	/* Determine if this client should be drawn. */
	if(!(np->statusFlags & STAT_MAPPED)) {
		return;
	}

	/* Determine the destop on which to draw it. */
	if(np->statusFlags & STAT_STICKY) {
		deskOffset = (pagerDeskWidth + 1) * currentDesktop;
	} else {
		deskOffset = (pagerDeskWidth + 1) * np->desktop;
	}

	/* Calculate the scaled coordinates to draw. */
	x = (int)((double)np->x * scalex);
	y = (int)((double)np->y * scaley);
	width = (int)((double)np->width * scalex);
	height = (int)((double)np->height * scaley);

	/* Check bounds. */
	if(x + width > pagerDeskWidth) {
		width = pagerDeskWidth - x;
	}
	if(x < 0) {
		width += x;
		x = 0;
	}
	if(width <= 0 || height <= 0) {
		return;
	}

	x += deskOffset;

	/* Draw. */
	XSetForeground(display, pagerGC, colors[COLOR_BORDER_TEXT]);
	XDrawRectangle(display, pager, pagerGC, x, y, width, height);

	if(width > 1 && height > 1) {
		if(np->statusFlags & STAT_ACTIVE
			&& (np->desktop == currentDesktop
			|| (np->statusFlags & STAT_STICKY))) {
			XSetForeground(display, pagerGC, colors[COLOR_ACTIVE_BORDER]);
		} else {
			XSetForeground(display, pagerGC, colors[COLOR_BORDER]);
		}
		XFillRectangle(display, pager, pagerGC, x + 1, y + 1,
			width - 1, height - 1);
	}

}


